use telers::{
    event::{telegram::HandlerResult, EventReturn},
    Bot,
};

use tokio::time::{sleep, Duration};

use crate::{
    types::{
        enums::target_user::TargetUser,
        structs::handler_entity::{ExtractedEntityData, HandlerEntity},
        TimeValues,
    },
    utils::{
        general::get_duration::{get_duration, ExtractedDuration},
        telegram::{restrict::restrict, try_do::try_restrict},
    },
};

pub async fn mute_member(
    handler_entity: HandlerEntity,
    chat_id: i64,
    user: TargetUser,
    time: TimeValues,
) -> HandlerResult {
    let (bot, message, mut sender_builder): ExtractedEntityData = handler_entity.extract();

    let user_id: i64 = user.get_id().unwrap();

    sleep(Duration::from_millis(time.1)).await;

    let (unmute_date, postfix, mute_duration): ExtractedDuration = get_duration(time);
    let demote_args: (&Bot, i64, i64) = (&bot, user_id, chat_id);

    let callback = || async { restrict(&bot, user_id, unmute_date, chat_id).await };

    sender_builder.set_text("Невозможно выдать мут.");

    if try_restrict(callback, demote_args, sender_builder.clone().build())
        .await
        .is_err()
    {
        Ok(EventReturn::Cancel)
    } else {
        let muted_user_name: String = user.get_user_name(&bot, &message).await.unwrap();

        sender_builder
            .reply_to(message.id())
            .text(format!(
                "Пользователю {muted_user_name} выдан мут на {mute_duration} {postfix}."
            ))
            .build()
            .send(&bot)
            .await?;

        Ok(EventReturn::Finish)
    }
}
